# Remix with [Prisma](https://www.prisma.io/) on Cloudflare

This example is for Cloudflare Workers, but it is basically the same for Cloudflare Pages.

In order to use Prisma on Cloudflare Workers, the [Prisma Data Proxy](https://www.prisma.io/docs/concepts/data-platform/data-proxy) must be used.  
However, the Remix compiler does not build correctly as it is because it prioritizes the browser when building code for Cloudflare.  
You need to devise a way to reference the code for cjs when resolving `@prisma/client`.

## Use remix-esbuild-override

Use remix-esbuild-override and plugins for esbuild.

0. Use Prisma Data Proxy

Please refer to [this document](https://www.prisma.io/docs/concepts/data-platform/data-proxy) to prepare for using Prisma Data Proxy.
When running `prisma generate`, be sure to specify the environment variable `PRISMA_CLIENT_ENGINE_TYPE='dataproxy'`.

1. Install and setup `postinstall`

```bash
npm install -D remix-esbuild-override esbuild-plugin-alias @esbuild-plugins/node-globals-polyfill
# or
yarn add -D remix-esbuild-override esbuild-plugin-alias @esbuild-plugins/node-globals-polyfill
```

Update `scripts > postinstall` in package.json.

```json
"scripts": {
  "postinstall": "remix-esbuild-override"
}
```

**Run `npm install` or `yarn install` again to run `postinstall`.**

2. Update remix.config.js

```js
const { withEsbuildOverride } = require("remix-esbuild-override");
const alias = require("esbuild-plugin-alias");
const GlobalsPolyfills =
  require("@esbuild-plugins/node-globals-polyfill").default;

withEsbuildOverride((option, { isServer }) => {
  option.plugins = [
    alias({
      "@prisma/client": require.resolve("@prisma/client"),
    }),
    GlobalsPolyfills(),
    ...option.plugins,
  ];

  return option;
});

/**
 * @type {import('@remix-run/dev').AppConfig}
 */
module.exports = {
  serverBuildTarget: "cloudflare-workers",
  server: "./server.js",
  devServerBroadcastDelay: 1000,
  ignoredRouteFiles: [".*"],
  // appDirectory: "app",
  // assetsBuildDirectory: "public/build",
  // serverBuildPath: "build/index.js",
  // publicPath: "/build/",
};
```

That's all.

**Note**  
Currently, when trying to run prisma on miniflare, an error occurs and the server does not start ([prisma#12356](https://github.com/prisma/prisma/issues/12356)).  
This problem should be resolved in the near future, but until then, you will need to patch the prisma data proxy code using [patch-package](https://github.com/ds300/patch-package).

patches/@prisma+client+3.12.0.patch

```patch
diff --git a/node_modules/@prisma/client/runtime/proxy.js b/node_modules/@prisma/client/runtime/proxy.js
index a4fb62e..856d4ed 100644
--- a/node_modules/@prisma/client/runtime/proxy.js
+++ b/node_modules/@prisma/client/runtime/proxy.js
@@ -90,7 +90,7 @@ ${(0,Ft.default)(e.toString(),2)}
 ${(0,Ft.default)(i.toString(),$r)}
 }`:an(i,this.inputType)).join(`,${n?" ":`
 `}`),n?0:$r)}${n?"":`
-`}]`}return`${t}: ${an(e,this.inputType)}`}}toString(){return this._toString(this.value,this.key)}collectErrors(){var t;if(!this.hasError)return[];let e=[];if(this.error){let r=typeof((t=this.inputType)==null?void 0:t.type)=="object"?`${this.inputType.type.name}${this.inputType.isList?"[]":""}`:void 0;e.push({error:this.error,path:[this.key],id:r})}return Array.isArray(this.value)&&e.push(...Bi(this.value,(r,n)=>(r==null?void 0:r.collectErrors)?r.collectErrors().map(i=>({...i,path:[this.key,n,...i.path]})):[])),this.value instanceof Je&&e.push(...this.value.collectErrors().map(r=>({...r,path:[this.key,...r.path]}))),e}},"Ne");d(ft,"Arg");function Li({dmmf:e,rootTypeName:t,rootField:r,select:n}){n||(n={});let i=t==="query"?e.queryType:e.mutationType,a={args:[],outputType:{isList:!1,type:i,location:"outputObjectTypes"},name:t},o=Di(e,{[r]:n},a,[t]);return new Rs(t,o)}y(Li,"xi");d(Li,"makeDocument");function Ui(e){return e}y(Ui,"Ti");d(Ui,"transformDocument");function Di(e,t,r,n){let i=r.outputType.type;return Object.entries(t).reduce((a,[o,l])=>{let f=i.fieldMap?i.fieldMap[o]:i.fields.find(E=>E.name===o);if(!f)return a.push(new Ue({name:o,children:[],error:{type:"invalidFieldName",modelName:i.name,providedName:o,didYouMean:Vr(o,i.fields.map(E=>E.name)),outputType:i}})),a;if(typeof l!="boolean"&&f.outputType.location==="scalar"&&f.name!=="executeRaw"&&f.name!=="queryRaw"&&f.name!=="runCommandRaw"&&i.name!=="Query"&&!o.startsWith("aggregate")&&f.name!=="count")return a.push(new Ue({name:o,children:[],error:{type:"invalidFieldType",modelName:i.name,fieldName:o,providedValue:l}})),a;if(l===!1)return a;let u={name:f.name,fields:f.args,constraints:{minNumFields:null,maxNumFields:null}},c=typeof l=="object"?gs(l,["include","select"]):void 0,p=c?Mr(c,u,[],typeof f=="string"?void 0:f.outputType.type):void 0,g=f.outputType.location==="outputObjectTypes";if(l){if(l.select&&l.include)a.push(new Ue({name:o,children:[new Ue({name:"include",args:new Je,error:{type:"includeAndSelect",field:f}})]}));else if(l.include){let E=Object.keys(l.include);if(E.length===0)return a.push(new Ue({name:o,children:[new Ue({name:"include",args:new Je,error:{type:"emptyInclude",field:f}})]})),a;if(f.outputType.location==="outputObjectTypes"){let P=f.outputType.type,x=P.fields.filter(R=>R.outputType.location==="outputObjectTypes").map(R=>R.name),A=E.filter(R=>!x.includes(R));if(A.length>0)return a.push(...A.map(R=>new Ue({name:R,children:[new Ue({name:R,args:new Je,error:{type:"invalidFieldName",modelName:P.name,outputType:P,providedName:R,didYouMean:Vr(R,x)||void 0,isInclude:!0,isIncludeScalar:P.fields.some(q=>q.name===R)}})]}))),a}}else if(l.select){let E=Object.values(l.select);if(E.length===0)return a.push(new Ue({name:o,children:[new Ue({name:"select",args:new Je,error:{type:"emptySelect",field:f}})]})),a;if(E.filter(P=>P).length===0)return a.push(new Ue({name:o,children:[new Ue({name:"select",args:new Je,error:{type:"noTrueSelect",field:f}})]})),a}}let v=g?Cs(e,f.outputType.type):null,b=v;l&&(l.select?b=l.select:l.include?b=rn(v,l.include):l.by&&Array.isArray(l.by)&&f.outputType.namespace==="prisma"&&f.outputType.location==="outputObjectTypes"&&ga(f.outputType.type.name)&&(b=Bs(l.by)));let w=b!==!1&&g?Di(e,b,f,[...n,o]):void 0;return a.push(new Ue({name:o,args:p,children:w,schemaField:f})),a},[])}y(Di,"fl");d(Di,"selectionToFields");function Bs(e){let t=Object.create(null);for(let r of e)t[r]=!0;return t}y(Bs,"rm");d(Bs,"byToSelect");function Cs(e,t){let r=Object.create(null);for(let n of t.fields)e.typeMap[n.outputType.type.name]!==void 0&&(r[n.name]=!0),(n.outputType.location==="scalar"||n.outputType.location==="enumTypes")&&(r[n.name]=!0);return r}y(Cs,"nm");d(Cs,"getDefaultSelection");function sn(e,t,r,n){return new ft({key:e,value:t,isEnum:n.location==="enumTypes",inputType:n,error:{type:"invalidType",providedValue:t,argName:e,requiredType:{inputType:r.inputTypes,bestFittingType:n}}})}y(sn,"Pi");d(sn,"getInvalidTypeArg");function Fs(e,t,r){let{type:n,isList:i}=r,a=vr(Ht(n),i),o=Zt(e,n);return!!(o===a||i&&o==="List<>"||a==="Json"||o==="Int"&&a==="BigInt"||o==="List<Int>"&&a==="List<BigInt>"||o==="List<BigInt | Int>"&&a==="List<BigInt>"||o==="List<Int | BigInt>"&&a==="List<BigInt>"||(o==="Int"||o==="Float")&&a==="Decimal"||(o==="List<Int>"||o==="List<Float>")&&a==="List<Decimal>"||o==="DateTime"&&a==="String"||o==="List<DateTime>"&&a==="List<String>"||o==="UUID"&&a==="String"||o==="List<UUID>"&&a==="List<String>"||o==="String"&&a==="ID"||o==="List<String>"&&a==="List<ID>"||o==="List<String>"&&a==="List<Json>"||a==="List<String>"&&(o==="List<String | UUID>"||o==="List<UUID | String>")||o==="Int"&&a==="Float"||o==="List<Int>"&&a==="List<Float>"||o==="Int"&&a==="Long"||o==="List<Int>"&&a==="List<Long>"||o==="String"&&a==="Decimal"&&/^\-?(\d+(\.\d*)?|\.\d+)(e[+-]?\d+)?$/i.test(e)||e===null)}y(Fs,"om");d(Fs,"hasCorrectScalarType");var un=d(e=>ps(e,(t,r)=>r!==void 0),"cleanObject");function qs(e,t,r){let n=null,i=[];for(let a of r.inputTypes){if(n=Us(e,t,r,a),(n==null?void 0:n.collectErrors().length)===0)return n;if(n&&(n==null?void 0:n.collectErrors())){let o=n==null?void 0:n.collectErrors();o&&o.length>0&&i.push({arg:n,errors:o})}}if((n==null?void 0:n.hasError)&&i.length>0){let a=i.map(({arg:o,errors:l})=>{let f=l.map(u=>{let c=1;return u.error.type==="invalidType"&&(c=2*Math.exp(Vi(u.error.providedValue))+1),c+=Math.log(u.path.length),u.error.type==="missingArg"&&o.inputType&&Lt(o.inputType.type)&&o.inputType.type.name.includes("Unchecked")&&(c*=2),u.error.type==="invalidName"&&Lt(u.error.originalType)&&u.error.originalType.name.includes("Unchecked")&&(c*=2),c});return{score:l.length+Ls(f),arg:o,errors:l}});return a.sort((o,l)=>o.score<l.score?-1:1),a[0].arg}return n}y(qs,"im");d(qs,"valueToArg");function Vi(e){let t=1;if(!e||typeof e!="object")return t;for(let r in e)if(!!Object.prototype.hasOwnProperty.call(e,r)&&typeof e[r]=="object"){let n=Vi(e[r])+1;t=Math.max(n,t)}return t}y(Vi,"pl");d(Vi,"getDepth");function Ls(e){return e.reduce((t,r)=>t+r,0)}y(Ls,"sm");d(Ls,"sum");function Us(e,t,r,n){var f,u,c,p;if(typeof t=="undefined")return r.isRequired?new ft({key:e,value:t,isEnum:n.location==="enumTypes",inputType:n,error:{type:"missingArg",missingName:e,missingArg:r,atLeastOne:!1,atMostOne:!1}}):null;let{isNullable:i,isRequired:a}=r;if(t===null&&!i&&!a&&!(Lt(n.type)?n.type.constraints.minNumFields!==null&&n.type.constraints.minNumFields>0:!1))return new ft({key:e,value:t,isEnum:n.location==="enumTypes",inputType:n,error:{type:"invalidNullArg",name:e,invalidType:r.inputTypes,atLeastOne:!1,atMostOne:!1}});if(!n.isList)if(Lt(n.type)){if(typeof t!="object"||Array.isArray(t)||n.location==="inputObjectTypes"&&!hs(t))return sn(e,t,r,n);{let g=un(t),v,b=Object.keys(g||{}),w=b.length;return w===0&&typeof n.type.constraints.minNumFields=="number"&&n.type.constraints.minNumFields>0?v={type:"atLeastOne",key:e,inputType:n.type}:w>1&&typeof n.type.constraints.maxNumFields=="number"&&n.type.constraints.maxNumFields<2&&(v={type:"atMostOne",key:e,inputType:n.type,providedKeys:b}),new ft({key:e,value:g===null?null:Mr(g,n.type,r.inputTypes),isEnum:n.location==="enumTypes",error:v,inputType:n,schemaArg:r})}}else return Gi(e,t,r,n);if(!Array.isArray(t)&&n.isList&&e!=="updateMany"&&(t=[t]),n.location==="enumTypes"||n.location==="scalar")return Gi(e,t,r,n);let o=n.type,l=(typeof((f=o.constraints)==null?void 0:f.minNumFields)=="number"&&((u=o.constraints)==null?void 0:u.minNumFields)>0?Array.isArray(t)&&t.some(g=>!g||Object.keys(un(g)).length===0):!1)?{inputType:o,key:e,type:"atLeastOne"}:void 0;if(!l){let g=typeof((c=o.constraints)==null?void 0:c.maxNumFields)=="number"&&((p=o.constraints)==null?void 0:p.maxNumFields)<2?Array.isArray(t)&&t.find(v=>!v||Object.keys(un(v)).length!==1):!1;g&&(l={inputType:o,key:e,type:"atMostOne",providedKeys:Object.keys(g)})}if(!Array.isArray(t))for(let g of r.inputTypes){let v=Mr(t,g.type);if(v.collectErrors().length===0)return new ft({key:e,value:v,isEnum:!1,schemaArg:r,inputType:g})}return new ft({key:e,value:t.map(g=>n.isList&&typeof g!="object"?g:typeof g!="object"||!t?sn(e,g,r,n):Mr(g,o)),isEnum:!1,inputType:n,schemaArg:r,error:l})}y(Us,"am");d(Us,"tryInferArgs");function Lt(e){return!(typeof e=="string"||Object.hasOwnProperty.call(e,"values"))}y(Lt,"Xt");d(Lt,"isInputArgType");function Gi(e,t,r,n){return Fs(t,r,n)?new ft({key:e,value:t,isEnum:n.location==="enumTypes",schemaArg:r,inputType:n}):sn(e,t,r,n)}y(Gi,"dl");d(Gi,"scalarToArg");function Mr(e,t,r,n){let i=un(e),{fields:a,fieldMap:o}=t,l=a.map(c=>[c.name,void 0]),f=Object.entries(i||{}),u=ha(f,l,c=>c[0]).reduce((c,[p,g])=>{let v=o?o[p]:a.find(w=>w.name===p);if(!v){let w=typeof g=="boolean"&&n&&n.fields.some(E=>E.name===p)?p:null;return c.push(new ft({key:p,value:g,error:{type:"invalidName",providedName:p,providedValue:g,didYouMeanField:w,didYouMeanArg:!w&&Vr(p,[...a.map(E=>E.name),"select"])||void 0,originalType:t,possibilities:r,outputType:n}})),c}let b=qs(p,g,v);return b&&c.push(b),c},[]);if(typeof t.constraints.minNumFields=="number"&&f.length<t.constraints.minNumFields||u.find(c=>{var p,g;return((p=c.error)==null?void 0:p.type)==="missingArg"||((g=c.error)==null?void 0:g.type)==="atLeastOne"})){let c=t.fields.filter(p=>!p.isRequired&&i&&(typeof i[p.name]=="undefined"||i[p.name]===null));u.push(...c.map(p=>{let g=p.inputTypes[0];return new ft({key:p.name,value:void 0,isEnum:g.location==="enumTypes",error:{type:"missingArg",missingName:p.name,missingArg:p,atLeastOne:Boolean(t.constraints.minNumFields)||!1,atMostOne:t.constraints.maxNumFields===1||!1},inputType:g})}))}return new Je(u)}y(Mr,"Jn");d(Mr,"objectToArgs");function Ji({document:e,path:t,data:r}){let n=Ka(r,t);if(n==="undefined")return null;if(typeof n!="object")return n;let i=Ds(e,t);return ln({field:i,data:n})}y(Ji,"Mi");d(Ji,"unpack");function ln({field:e,data:t}){var n;if(!t||typeof t!="object"||!e.children||!e.schemaField)return t;let r={DateTime:i=>new Date(i),Json:i=>JSON.parse(i),Bytes:i=>Ge.Buffer.from(i,"base64"),Decimal:i=>new Dr(i),BigInt:i=>BigInt(i)};for(let i of e.children){let a=(n=i.schemaField)==null?void 0:n.outputType.type;if(a&&typeof a=="string"){let o=r[a];if(o)if(Array.isArray(t))for(let l of t)typeof l[i.name]!="undefined"&&l[i.name]!==null&&(Array.isArray(l[i.name])?l[i.name]=l[i.name].map(o):l[i.name]=o(l[i.name]));else typeof t[i.name]!="undefined"&&t[i.name]!==null&&(Array.isArray(t[i.name])?t[i.name]=t[i.name].map(o):t[i.name]=o(t[i.name]))}if(i.schemaField&&i.schemaField.outputType.location==="outputObjectTypes")if(Array.isArray(t))for(let o of t)ln({field:i,data:o[i.name]});else ln({field:i,data:t[i.name]})}return t}y(ln,"Oi");d(ln,"mapScalars");function Ds(e,t){let r=t.slice(),n=r.shift(),i=e.children.find(a=>a.name===n);if(!i)throw new Error(`Could not find field ${n} in document ${e}`);for(;r.length>0;){let a=r.shift();if(!i.children)throw new Error(`Can't get children for field ${i} with child ${a}`);let o=i.children.find(l=>l.name===a);if(!o)throw new Error(`Can't find child ${a} of field ${i}`);i=o}return i}y(Ds,"um");d(Ds,"getField");function cn(e){return e.split(".").filter(t=>t!=="select").join(".")}y(cn,"Si");d(cn,"removeSelectFromPath");function fn(e){if(Object.prototype.toString.call(e)==="[object Object]"){let t={};for(let r in e)if(r==="select")for(let n in e.select)t[n]=fn(e.select[n]);else t[r]=fn(e[r]);return t}return e}y(fn,"Ii");d(fn,"removeSelectFromObject");function Vs({ast:e,keyPaths:t,missingItems:r,valuePaths:n}){let i=t.map(cn),a=n.map(cn),o=r.map(l=>({path:cn(l.path),isRequired:l.isRequired,type:l.type}));return{ast:fn(e),keyPaths:i,missingItems:o,valuePaths:a}}y(Vs,"lm");d(Vs,"transformAggregatePrintJsonArgs");j();k();N();I();var zc=Ae(co()),Yc=Ae(xn());j();k();N();I();var Gs=y(class{constructor(e){this.options=e,this.tickActive=!1,this.batches={}}request(e){let t=this.options.batchBy(e);return t?(this.batches[t]||(this.batches[t]=[],this.tickActive||(this.tickActive=!0,we.nextTick(()=>{this.dispatchBatches(),this.tickActive=!1}))),new Promise((r,n)=>{this.batches[t].push({request:e,resolve:r,reject:n})})):this.options.singleLoader(e)}dispatchBatches(){for(let e in this.batches){let t=this.batches[e];delete this.batches[e],t.length===1?this.options.singleLoader(t[0].request).then(r=>{r instanceof Error?t[0].reject(r):t[0].resolve(r)}).catch(r=>{t[0].reject(r)}):this.options.batchLoader(t.map(r=>r.request)).then(r=>{if(r instanceof Error)for(let n=0;n<t.length;n++)t[n].reject(r);else for(let n=0;n<t.length;n++){let i=r[n];i instanceof Error?t[n].reject(i):t[n].resolve(i)}}).catch(r=>{for(let n=0;n<t.length;n++)t[n].reject(r)})}}get[Symbol.toStringTag](){return"DataLoader"}},"zn");d(Gs,"DataLoader");j();k();N();I();var pn=y(class extends Error{constructor(e){super(e);this.name="NotFoundError"}},"Ur");d(pn,"NotFoundError");function Wi(e,t,r,n){let i;if(r&&typeof r=="object"&&"rejectOnNotFound"in r&&r.rejectOnNotFound!==void 0)i=r.rejectOnNotFound,delete r.rejectOnNotFound;else if(typeof n=="boolean")i=n;else if(n&&typeof n=="object"&&e in n){let a=n[e];if(a&&typeof a=="object")return t in a?a[t]:void 0;i=Wi(e,t,r,a)}else typeof n=="function"?i=n:i=!1;return i}y(Wi,"ki");d(Wi,"getRejectOnNotFound");var Hc=/(findUnique|findFirst)/;function Js(e,t,r,n){if(n&&!e&&Hc.exec(t))throw typeof n=="boolean"&&n?new pn(`No ${r} found`):typeof n=="function"?n(new pn(`No ${r} found`)):Zr(n)?n:new pn(`No ${r} found`)}y(Js,"gl");d(Js,"throwIfNotFound");var Zc=(0,zc.default)("prisma:client:request_handler");function zi(e){let t=e[0].transactionId,r=e[0].runInTransaction,n=e[0].headers;return{inTx:typeof t=="number"&&r?!0:void 0,headers:{transactionId:typeof t=="string"&&r?t:void 0,...n}}}y(zi,"yl");d(zi,"getRequestInfo");var Ws=y(class{constructor(e,t){this.client=e,this.hooks=t,this.dataloader=new Gs({batchLoader:r=>{let n=zi(r),i=r.map(a=>String(a.document));return this.client._engine.requestBatch(i,n.headers,n.inTx)},singleLoader:r=>{let n=zi([r]),i=String(r.document);return this.client._engine.request(i,n.headers)},batchBy:r=>r.transactionId?`transaction-${r.transactionId}`:zs(r)})}async request({document:e,dataPath:t=[],rootField:r,typeName:n,isList:i,callsite:a,rejectOnNotFound:o,clientMethod:l,runInTransaction:f,showColors:u,engineHook:c,args:p,headers:g,transactionId:v,unpacker:b}){if(this.hooks&&this.hooks.beforeRequest){let w=String(e);this.hooks.beforeRequest({query:w,path:t,rootField:r,typeName:n,document:e,isList:i,clientMethod:l,args:p})}try{let w,E;if(c){let x=await c({document:e,runInTransaction:f},A=>this.dataloader.request(A));w=x.data,E=x.elapsed}else{let x=await this.dataloader.request({document:e,runInTransaction:f,headers:g,transactionId:v});w=x==null?void 0:x.data,E=x==null?void 0:x.elapsed}let P=this.unpack(e,w,t,r,b);return Js(P,l,n,o),we.env.PRISMA_CLIENT_GET_TIME?{data:P,elapsed:E}:P}catch(w){Zc(w);let E=w.message;if(a){let{stack:P}=Ms({callsite:a,originalMethod:l,onUs:w.isPanic,showColors:u});E=`${P}
+`}]`}return`${t}: ${an(e,this.inputType)}`}}toString(){return this._toString(this.value,this.key)}collectErrors(){var t;if(!this.hasError)return[];let e=[];if(this.error){let r=typeof((t=this.inputType)==null?void 0:t.type)=="object"?`${this.inputType.type.name}${this.inputType.isList?"[]":""}`:void 0;e.push({error:this.error,path:[this.key],id:r})}return Array.isArray(this.value)&&e.push(...Bi(this.value,(r,n)=>(r==null?void 0:r.collectErrors)?r.collectErrors().map(i=>({...i,path:[this.key,n,...i.path]})):[])),this.value instanceof Je&&e.push(...this.value.collectErrors().map(r=>({...r,path:[this.key,...r.path]}))),e}},"Ne");d(ft,"Arg");function Li({dmmf:e,rootTypeName:t,rootField:r,select:n}){n||(n={});let i=t==="query"?e.queryType:e.mutationType,a={args:[],outputType:{isList:!1,type:i,location:"outputObjectTypes"},name:t},o=Di(e,{[r]:n},a,[t]);return new Rs(t,o)}y(Li,"xi");d(Li,"makeDocument");function Ui(e){return e}y(Ui,"Ti");d(Ui,"transformDocument");function Di(e,t,r,n){let i=r.outputType.type;return Object.entries(t).reduce((a,[o,l])=>{let f=i.fieldMap?i.fieldMap[o]:i.fields.find(E=>E.name===o);if(!f)return a.push(new Ue({name:o,children:[],error:{type:"invalidFieldName",modelName:i.name,providedName:o,didYouMean:Vr(o,i.fields.map(E=>E.name)),outputType:i}})),a;if(typeof l!="boolean"&&f.outputType.location==="scalar"&&f.name!=="executeRaw"&&f.name!=="queryRaw"&&f.name!=="runCommandRaw"&&i.name!=="Query"&&!o.startsWith("aggregate")&&f.name!=="count")return a.push(new Ue({name:o,children:[],error:{type:"invalidFieldType",modelName:i.name,fieldName:o,providedValue:l}})),a;if(l===!1)return a;let u={name:f.name,fields:f.args,constraints:{minNumFields:null,maxNumFields:null}},c=typeof l=="object"?gs(l,["include","select"]):void 0,p=c?Mr(c,u,[],typeof f=="string"?void 0:f.outputType.type):void 0,g=f.outputType.location==="outputObjectTypes";if(l){if(l.select&&l.include)a.push(new Ue({name:o,children:[new Ue({name:"include",args:new Je,error:{type:"includeAndSelect",field:f}})]}));else if(l.include){let E=Object.keys(l.include);if(E.length===0)return a.push(new Ue({name:o,children:[new Ue({name:"include",args:new Je,error:{type:"emptyInclude",field:f}})]})),a;if(f.outputType.location==="outputObjectTypes"){let P=f.outputType.type,x=P.fields.filter(R=>R.outputType.location==="outputObjectTypes").map(R=>R.name),A=E.filter(R=>!x.includes(R));if(A.length>0)return a.push(...A.map(R=>new Ue({name:R,children:[new Ue({name:R,args:new Je,error:{type:"invalidFieldName",modelName:P.name,outputType:P,providedName:R,didYouMean:Vr(R,x)||void 0,isInclude:!0,isIncludeScalar:P.fields.some(q=>q.name===R)}})]}))),a}}else if(l.select){let E=Object.values(l.select);if(E.length===0)return a.push(new Ue({name:o,children:[new Ue({name:"select",args:new Je,error:{type:"emptySelect",field:f}})]})),a;if(E.filter(P=>P).length===0)return a.push(new Ue({name:o,children:[new Ue({name:"select",args:new Je,error:{type:"noTrueSelect",field:f}})]})),a}}let v=g?Cs(e,f.outputType.type):null,b=v;l&&(l.select?b=l.select:l.include?b=rn(v,l.include):l.by&&Array.isArray(l.by)&&f.outputType.namespace==="prisma"&&f.outputType.location==="outputObjectTypes"&&ga(f.outputType.type.name)&&(b=Bs(l.by)));let w=b!==!1&&g?Di(e,b,f,[...n,o]):void 0;return a.push(new Ue({name:o,args:p,children:w,schemaField:f})),a},[])}y(Di,"fl");d(Di,"selectionToFields");function Bs(e){let t=Object.create(null);for(let r of e)t[r]=!0;return t}y(Bs,"rm");d(Bs,"byToSelect");function Cs(e,t){let r=Object.create(null);for(let n of t.fields)e.typeMap[n.outputType.type.name]!==void 0&&(r[n.name]=!0),(n.outputType.location==="scalar"||n.outputType.location==="enumTypes")&&(r[n.name]=!0);return r}y(Cs,"nm");d(Cs,"getDefaultSelection");function sn(e,t,r,n){return new ft({key:e,value:t,isEnum:n.location==="enumTypes",inputType:n,error:{type:"invalidType",providedValue:t,argName:e,requiredType:{inputType:r.inputTypes,bestFittingType:n}}})}y(sn,"Pi");d(sn,"getInvalidTypeArg");function Fs(e,t,r){let{type:n,isList:i}=r,a=vr(Ht(n),i),o=Zt(e,n);return!!(o===a||i&&o==="List<>"||a==="Json"||o==="Int"&&a==="BigInt"||o==="List<Int>"&&a==="List<BigInt>"||o==="List<BigInt | Int>"&&a==="List<BigInt>"||o==="List<Int | BigInt>"&&a==="List<BigInt>"||(o==="Int"||o==="Float")&&a==="Decimal"||(o==="List<Int>"||o==="List<Float>")&&a==="List<Decimal>"||o==="DateTime"&&a==="String"||o==="List<DateTime>"&&a==="List<String>"||o==="UUID"&&a==="String"||o==="List<UUID>"&&a==="List<String>"||o==="String"&&a==="ID"||o==="List<String>"&&a==="List<ID>"||o==="List<String>"&&a==="List<Json>"||a==="List<String>"&&(o==="List<String | UUID>"||o==="List<UUID | String>")||o==="Int"&&a==="Float"||o==="List<Int>"&&a==="List<Float>"||o==="Int"&&a==="Long"||o==="List<Int>"&&a==="List<Long>"||o==="String"&&a==="Decimal"&&/^\-?(\d+(\.\d*)?|\.\d+)(e[+-]?\d+)?$/i.test(e)||e===null)}y(Fs,"om");d(Fs,"hasCorrectScalarType");var un=d(e=>ps(e,(t,r)=>r!==void 0),"cleanObject");function qs(e,t,r){let n=null,i=[];for(let a of r.inputTypes){if(n=Us(e,t,r,a),(n==null?void 0:n.collectErrors().length)===0)return n;if(n&&(n==null?void 0:n.collectErrors())){let o=n==null?void 0:n.collectErrors();o&&o.length>0&&i.push({arg:n,errors:o})}}if((n==null?void 0:n.hasError)&&i.length>0){let a=i.map(({arg:o,errors:l})=>{let f=l.map(u=>{let c=1;return u.error.type==="invalidType"&&(c=2*Math.exp(Vi(u.error.providedValue))+1),c+=Math.log(u.path.length),u.error.type==="missingArg"&&o.inputType&&Lt(o.inputType.type)&&o.inputType.type.name.includes("Unchecked")&&(c*=2),u.error.type==="invalidName"&&Lt(u.error.originalType)&&u.error.originalType.name.includes("Unchecked")&&(c*=2),c});return{score:l.length+Ls(f),arg:o,errors:l}});return a.sort((o,l)=>o.score<l.score?-1:1),a[0].arg}return n}y(qs,"im");d(qs,"valueToArg");function Vi(e){let t=1;if(!e||typeof e!="object")return t;for(let r in e)if(!!Object.prototype.hasOwnProperty.call(e,r)&&typeof e[r]=="object"){let n=Vi(e[r])+1;t=Math.max(n,t)}return t}y(Vi,"pl");d(Vi,"getDepth");function Ls(e){return e.reduce((t,r)=>t+r,0)}y(Ls,"sm");d(Ls,"sum");function Us(e,t,r,n){var f,u,c,p;if(typeof t=="undefined")return r.isRequired?new ft({key:e,value:t,isEnum:n.location==="enumTypes",inputType:n,error:{type:"missingArg",missingName:e,missingArg:r,atLeastOne:!1,atMostOne:!1}}):null;let{isNullable:i,isRequired:a}=r;if(t===null&&!i&&!a&&!(Lt(n.type)?n.type.constraints.minNumFields!==null&&n.type.constraints.minNumFields>0:!1))return new ft({key:e,value:t,isEnum:n.location==="enumTypes",inputType:n,error:{type:"invalidNullArg",name:e,invalidType:r.inputTypes,atLeastOne:!1,atMostOne:!1}});if(!n.isList)if(Lt(n.type)){if(typeof t!="object"||Array.isArray(t)||n.location==="inputObjectTypes"&&!hs(t))return sn(e,t,r,n);{let g=un(t),v,b=Object.keys(g||{}),w=b.length;return w===0&&typeof n.type.constraints.minNumFields=="number"&&n.type.constraints.minNumFields>0?v={type:"atLeastOne",key:e,inputType:n.type}:w>1&&typeof n.type.constraints.maxNumFields=="number"&&n.type.constraints.maxNumFields<2&&(v={type:"atMostOne",key:e,inputType:n.type,providedKeys:b}),new ft({key:e,value:g===null?null:Mr(g,n.type,r.inputTypes),isEnum:n.location==="enumTypes",error:v,inputType:n,schemaArg:r})}}else return Gi(e,t,r,n);if(!Array.isArray(t)&&n.isList&&e!=="updateMany"&&(t=[t]),n.location==="enumTypes"||n.location==="scalar")return Gi(e,t,r,n);let o=n.type,l=(typeof((f=o.constraints)==null?void 0:f.minNumFields)=="number"&&((u=o.constraints)==null?void 0:u.minNumFields)>0?Array.isArray(t)&&t.some(g=>!g||Object.keys(un(g)).length===0):!1)?{inputType:o,key:e,type:"atLeastOne"}:void 0;if(!l){let g=typeof((c=o.constraints)==null?void 0:c.maxNumFields)=="number"&&((p=o.constraints)==null?void 0:p.maxNumFields)<2?Array.isArray(t)&&t.find(v=>!v||Object.keys(un(v)).length!==1):!1;g&&(l={inputType:o,key:e,type:"atMostOne",providedKeys:Object.keys(g)})}if(!Array.isArray(t))for(let g of r.inputTypes){let v=Mr(t,g.type);if(v.collectErrors().length===0)return new ft({key:e,value:v,isEnum:!1,schemaArg:r,inputType:g})}return new ft({key:e,value:t.map(g=>n.isList&&typeof g!="object"?g:typeof g!="object"||!t?sn(e,g,r,n):Mr(g,o)),isEnum:!1,inputType:n,schemaArg:r,error:l})}y(Us,"am");d(Us,"tryInferArgs");function Lt(e){return!(typeof e=="string"||Object.hasOwnProperty.call(e,"values"))}y(Lt,"Xt");d(Lt,"isInputArgType");function Gi(e,t,r,n){return Fs(t,r,n)?new ft({key:e,value:t,isEnum:n.location==="enumTypes",schemaArg:r,inputType:n}):sn(e,t,r,n)}y(Gi,"dl");d(Gi,"scalarToArg");function Mr(e,t,r,n){let i=un(e),{fields:a,fieldMap:o}=t,l=a.map(c=>[c.name,void 0]),f=Object.entries(i||{}),u=ha(f,l,c=>c[0]).reduce((c,[p,g])=>{let v=o?o[p]:a.find(w=>w.name===p);if(!v){let w=typeof g=="boolean"&&n&&n.fields.some(E=>E.name===p)?p:null;return c.push(new ft({key:p,value:g,error:{type:"invalidName",providedName:p,providedValue:g,didYouMeanField:w,didYouMeanArg:!w&&Vr(p,[...a.map(E=>E.name),"select"])||void 0,originalType:t,possibilities:r,outputType:n}})),c}let b=qs(p,g,v);return b&&c.push(b),c},[]);if(typeof t.constraints.minNumFields=="number"&&f.length<t.constraints.minNumFields||u.find(c=>{var p,g;return((p=c.error)==null?void 0:p.type)==="missingArg"||((g=c.error)==null?void 0:g.type)==="atLeastOne"})){let c=t.fields.filter(p=>!p.isRequired&&i&&(typeof i[p.name]=="undefined"||i[p.name]===null));u.push(...c.map(p=>{let g=p.inputTypes[0];return new ft({key:p.name,value:void 0,isEnum:g.location==="enumTypes",error:{type:"missingArg",missingName:p.name,missingArg:p,atLeastOne:Boolean(t.constraints.minNumFields)||!1,atMostOne:t.constraints.maxNumFields===1||!1},inputType:g})}))}return new Je(u)}y(Mr,"Jn");d(Mr,"objectToArgs");function Ji({document:e,path:t,data:r}){let n=Ka(r,t);if(n==="undefined")return null;if(typeof n!="object")return n;let i=Ds(e,t);return ln({field:i,data:n})}y(Ji,"Mi");d(Ji,"unpack");function ln({field:e,data:t}){var n;if(!t||typeof t!="object"||!e.children||!e.schemaField)return t;let r={DateTime:i=>new Date(i),Json:i=>JSON.parse(i),Bytes:i=>Ge.Buffer.from(i,"base64"),Decimal:i=>new Dr(i),BigInt:i=>BigInt(i)};for(let i of e.children){let a=(n=i.schemaField)==null?void 0:n.outputType.type;if(a&&typeof a=="string"){let o=r[a];if(o)if(Array.isArray(t))for(let l of t)typeof l[i.name]!="undefined"&&l[i.name]!==null&&(Array.isArray(l[i.name])?l[i.name]=l[i.name].map(o):l[i.name]=o(l[i.name]));else typeof t[i.name]!="undefined"&&t[i.name]!==null&&(Array.isArray(t[i.name])?t[i.name]=t[i.name].map(o):t[i.name]=o(t[i.name]))}if(i.schemaField&&i.schemaField.outputType.location==="outputObjectTypes")if(Array.isArray(t))for(let o of t)ln({field:i,data:o[i.name]});else ln({field:i,data:t[i.name]})}return t}y(ln,"Oi");d(ln,"mapScalars");function Ds(e,t){let r=t.slice(),n=r.shift(),i=e.children.find(a=>a.name===n);if(!i)throw new Error(`Could not find field ${n} in document ${e}`);for(;r.length>0;){let a=r.shift();if(!i.children)throw new Error(`Can't get children for field ${i} with child ${a}`);let o=i.children.find(l=>l.name===a);if(!o)throw new Error(`Can't find child ${a} of field ${i}`);i=o}return i}y(Ds,"um");d(Ds,"getField");function cn(e){return e.split(".").filter(t=>t!=="select").join(".")}y(cn,"Si");d(cn,"removeSelectFromPath");function fn(e){if(Object.prototype.toString.call(e)==="[object Object]"){let t={};for(let r in e)if(r==="select")for(let n in e.select)t[n]=fn(e.select[n]);else t[r]=fn(e[r]);return t}return e}y(fn,"Ii");d(fn,"removeSelectFromObject");function Vs({ast:e,keyPaths:t,missingItems:r,valuePaths:n}){let i=t.map(cn),a=n.map(cn),o=r.map(l=>({path:cn(l.path),isRequired:l.isRequired,type:l.type}));return{ast:fn(e),keyPaths:i,missingItems:o,valuePaths:a}}y(Vs,"lm");d(Vs,"transformAggregatePrintJsonArgs");j();k();N();I();var zc=Ae(co()),Yc=Ae(xn());j();k();N();I();var Gs=y(class{constructor(e){this.options=e,this.tickActive=!1,this.batches={}}request(e){let t=this.options.batchBy(e);return t?(this.batches[t]||(this.batches[t]=[],this.tickActive||(this.tickActive=!0,we.nextTick(()=>{this.dispatchBatches(),this.tickActive=!1}))),new Promise((r,n)=>{this.batches[t].push({request:e,resolve:r,reject:n})})):this.options.singleLoader(e)}dispatchBatches(){for(let e in this.batches){let t=this.batches[e];delete this.batches[e],t.length===1?this.options.singleLoader(t[0].request).then(r=>{r instanceof Error?t[0].reject(r):t[0].resolve(r)}).catch(r=>{t[0].reject(r)}):this.options.batchLoader(t.map(r=>r.request)).then(r=>{if(r instanceof Error)for(let n=0;n<t.length;n++)t[n].reject(r);else for(let n=0;n<t.length;n++){let i=r[n];i instanceof Error?t[n].reject(i):t[n].resolve(i)}}).catch(r=>{for(let n=0;n<t.length;n++)t[n].reject(r)})}}get[Symbol.toStringTag](){return"DataLoader"}},"zn");d(Gs,"DataLoader");j();k();N();I();var pn=y(class extends Error{constructor(e){super(e);this.name="NotFoundError"}},"Ur");d(pn,"NotFoundError");function Wi(e,t,r,n){let i;if(r&&typeof r=="object"&&"rejectOnNotFound"in r&&r.rejectOnNotFound!==void 0)i=r.rejectOnNotFound,delete r.rejectOnNotFound;else if(typeof n=="boolean")i=n;else if(n&&typeof n=="object"&&e in n){let a=n[e];if(a&&typeof a=="object")return t in a?a[t]:void 0;i=Wi(e,t,r,a)}else typeof n=="function"?i=n:i=!1;return i}y(Wi,"ki");d(Wi,"getRejectOnNotFound");var Hc=/(findUnique|findFirst)/;function Js(e,t,r,n){if(n&&!e&&Hc.exec(t))throw typeof n=="boolean"&&n?new pn(`No ${r} found`):typeof n=="function"?n(new pn(`No ${r} found`)):Zr(n)?n:new pn(`No ${r} found`)}y(Js,"gl");d(Js,"throwIfNotFound");var Zc=(0,zc.default)("prisma:client:request_handler");function zi(e){let t=e[0].transactionId,r=e[0].runInTransaction,n=e[0].headers;const headers={...n};if(typeof t=="string"&&r){headers.transactionId= t}return{inTx:typeof t=="number"&&r?!0:void 0,headers}}y(zi,"yl");d(zi,"getRequestInfo");var Ws=y(class{constructor(e,t){this.client=e,this.hooks=t,this.dataloader=new Gs({batchLoader:r=>{let n=zi(r),i=r.map(a=>String(a.document));return this.client._engine.requestBatch(i,n.headers,n.inTx)},singleLoader:r=>{let n=zi([r]),i=String(r.document);return this.client._engine.request(i,n.headers)},batchBy:r=>r.transactionId?`transaction-${r.transactionId}`:zs(r)})}async request({document:e,dataPath:t=[],rootField:r,typeName:n,isList:i,callsite:a,rejectOnNotFound:o,clientMethod:l,runInTransaction:f,showColors:u,engineHook:c,args:p,headers:g,transactionId:v,unpacker:b}){if(this.hooks&&this.hooks.beforeRequest){let w=String(e);this.hooks.beforeRequest({query:w,path:t,rootField:r,typeName:n,document:e,isList:i,clientMethod:l,args:p})}try{let w,E;if(c){let x=await c({document:e,runInTransaction:f},A=>this.dataloader.request(A));w=x.data,E=x.elapsed}else{let x=await this.dataloader.request({document:e,runInTransaction:f,headers:g,transactionId:v});w=x==null?void 0:x.data,E=x==null?void 0:x.elapsed}let P=this.unpack(e,w,t,r,b);return Js(P,l,n,o),we.env.PRISMA_CLIENT_GET_TIME?{data:P,elapsed:E}:P}catch(w){Zc(w);let E=w.message;if(a){let{stack:P}=Ms({callsite:a,originalMethod:l,onUs:w.isPanic,showColors:u});E=`${P}
   ${w.message}`}throw E=this.sanitizeMessage(E),w.code?new wr(E,w.code,this.client._clientVersion,w.meta):w.isPanic?new er(E,this.client._clientVersion):w instanceof tr?new tr(E,this.client._clientVersion):w instanceof Xt?new Xt(E,this.client._clientVersion):w instanceof er?new er(E,this.client._clientVersion):(w.clientVersion=this.client._clientVersion,w)}}sanitizeMessage(e){return this.client._errorFormat&&this.client._errorFormat!=="pretty"?(0,Yc.default)(e):e}unpack(e,t,r,n,i){(t==null?void 0:t.data)&&(t=t.data),i&&(t[n]=i(t[n]));let a=[];return n&&a.push(n),a.push(...r.filter(o=>o!=="select"&&o!=="include")),Ji({document:e,data:t,path:a})}get[Symbol.toStringTag](){return"RequestHandler"}},"Wn");d(Ws,"RequestHandler");function zs(e){var n;if(!e.document.children[0].name.startsWith("findUnique"))return;let t=(n=e.document.children[0].args)==null?void 0:n.args.map(i=>i.value instanceof Je?`${i.key}-${i.value.args.map(a=>a.key).join(",")}`:i.key).join(","),r=e.document.children[0].children.join(",");return`${e.document.children[0].name}|${t}|${r}`}y(zs,"pm");d(zs,"batchFindUniqueBy");j();k();N();I();var Kc=Mu().version;j();k();N();I();var dn=d(e=>e.reduce((t,r,n)=>`${t}@P${n}${r}`),"mssqlPreparedStatement");j();k();N();I();function Ys(e,t){let r=t&&Ti.getSpanContext(t);return(r==null?void 0:r.traceFlags)===1?{traceparent:`00-${r.traceId}-${r.spanId}-01`,...e}:e!=null?e:{}}y(Ys,"vl");d(Ys,"applyTracingHeaders");j();k();N();I();async function Hs(e,t,r){if(t===void 0)return r(void 0);let n=Ti.getTracer("prisma").startSpan(e,void 0,t),i=Ti.setSpan(t,n),a=await Da.with(i,()=>r(n));return n==null||n.end(),a}y(Hs,"El");d(Hs,"runInChildSpan");j();k();N();I();function ot(e){return JSON.stringify(gn(hn(e)))}y(ot,"Ce");d(ot,"serializeRawParameters");function hn(e){let t=Object.prototype.toString.call(e);if(t==="[object Date]")return{prisma__type:"date",prisma__value:e.toJSON()};if(t==="[object Object]"){let r={};for(let n in e)n!=="__proto__"&&(r[n]=hn(e[n]));return r}if(t==="[object Array]"){let r=e.length,n;for(n=new Array(r);r--;)n[r]=hn(e[r]);return n}return e}y(hn,"Fi");d(hn,"replaceDates");function gn(e){let t=Object.prototype.toString.call(e);if(t==="[object BigInt]")return e.toString();if(t==="[object Object]"){let r={};for(let n in e)n!=="__proto__"&&(r[n]=gn(e[n]));return r}if(t==="[object Array]"){let r=e.length,n;for(n=new Array(r);r--;)n[r]=gn(e[r]);return n}return e}y(gn,"ji");d(gn,"serializeBigInt");j();k();N();I();var Qc=Ae(uo()),Zs=["datasources","errorFormat","log","__internal","rejectOnNotFound"],Ks=["pretty","colorless","minimal"],Qs=["info","query","warn","error"],Xc={datasources:(e,t)=>{if(e){if(typeof e!="object"||Array.isArray(e))throw new Re(`Invalid value ${JSON.stringify(e)} for "datasources" provided to PrismaClient constructor`);for(let[r,n]of Object.entries(e)){if(!t.includes(r)){let i=Ut(r,t)||`Available datasources: ${t.join(", ")}`;throw new Re(`Unknown datasource ${r} provided to PrismaClient constructor.${i}`)}if(typeof n!="object"||Array.isArray(n))throw new Re(`Invalid value ${JSON.stringify(e)} for datasource "${r}" provided to PrismaClient constructor.
 It should have this form: { url: "CONNECTION_STRING" }`);if(n&&typeof n=="object")for(let[i,a]of Object.entries(n)){if(i!=="url")throw new Re(`Invalid value ${JSON.stringify(e)} for datasource "${r}" provided to PrismaClient constructor.
 It should have this form: { url: "CONNECTION_STRING" }`);if(typeof a!="string")throw new Re(`Invalid value ${JSON.stringify(a)} for datasource "${r}" provided to PrismaClient constructor.
```

---

# Welcome to Remix!

- [Remix Docs](https://remix.run/docs)

## Development

You will be running two processes during development:

- The Miniflare server (miniflare is a local environment for Cloudflare Workers)
- The Remix development server

Both are started with one command:

```sh
npm run dev
```

Open up [http://127.0.0.1:8787](http://127.0.0.1:8787) and you should be ready to go!

If you want to check the production build, you can stop the dev server and run following commands:

```sh
npm run build
npm start
```

Then refresh the same URL in your browser (no live reload for production builds).

## Deployment

Use [wrangler](https://developers.cloudflare.com/workers/cli-wrangler) to build and deploy your application to Cloudflare Workers. If you don't have it yet, follow [the installation guide](https://developers.cloudflare.com/workers/cli-wrangler/install-update) to get it setup. Be sure to [authenticate the CLI](https://developers.cloudflare.com/workers/cli-wrangler/authentication) as well.

If you don't already have an account, then [create a cloudflare account here](https://dash.cloudflare.com/sign-up) and after verifying your email address with Cloudflare, go to your dashboard and set up your free custom Cloudflare Workers subdomain.

Once that's done, you should be able to deploy your app:

```sh
npm run deploy
```
